------------------------------------
-- Loatheb, originally by Conquest/Overrated, updated by Lowanaera of Heist
------------------------------------
local loatheb_track
local loatheb_sigupdate
local loatheb_doomcount
local loatheb_sporegroup
local loatheb_sporelockout


function RDXM.NAXE.LoathebActivate()
	VFLEvent:NamedBind("loatheb", BlizzEvent("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE"), function() RDXM.NAXE.LoathebSpore(arg1); end);
	VFLEvent:NamedBind("loatheb", BlizzEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE"), function() RDXM.NAXE.LoathebSpore(arg1); end);
	VFLEvent:NamedBind("loatheb", BlizzEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE"), function() RDXM.NAXE.LoathebSpore(arg1); end);

	RPC.Bind("loatheb_spore", RDXM.NAXE.LoathebSporeAlert);

	-- Tracking
	loatheb_track = HOT.TrackTarget("Loatheb");
	loatheb_sigupdate = loatheb_track.SigUpdate:Connect(RDXM.NAXE, "LoathebUpdate");

	if IsRaidOfficer() or IsRaidLeader() then
		RDXM.BossMods.SetThreatMasterTarget("Loatheb", 2)
	end
end

function RDXM.NAXE.LoathebDeactivate()
	-- Unbind Events
	VFLEvent:NamedUnbind("loatheb");
	
	RPC.UnbindPattern("^loatheb");

	if loatheb_track then
		loatheb_track.SigUpdate:DisconnectByHandle(loatheb_track);
		loatheb_sigupdate = nil; loatheb_track = nil;
	end
end

function RDXM.NAXE.LoathebSetupHealerOrder()
	if loatheb_healers then
		RPC.Invoke("loatheb_healerorder", loatheb_healers);
	end
end

function RDXM.NAXE.LoathebSetupHealerChannel()
	if loatheb_healerchannel then
		RPC.Invoke("loatheb_healerchannel", loatheb_healerchannel);
	end
end

function RDXM.NAXE.LoathebStart()
	loatheb_doomcount = 1;
    loatheb_sporegroup = 1
    loatheb_sporelockout = nil
	VFL.scheduleExclusive("loatheb_doom", 120, function() RDXM.NAXE.LoathebStartDoomTimer(); end);
	RDX.Alert.Dropdown("loatheb_doom", "Doom - Shadow Pot", 130, 5, nil, nil, {r=1, g=.2, b=.2});
	if (IsRaidLeader()) then
		RDXM.NAXE.LoathebSetupHealerOrder();
		RDXM.NAXE.LoathebSetupHealerChannel();
	end
end

function RDXM.NAXE.LoathebStop()
	--remove any lingering alerts
	RDX.QuashAlertsByPattern("^loatheb")

	--unschedule everything else
	VFL.removeScheduledEventByName("loatheb_doom");
	VFL.removeScheduledEventByName("loatheb_healstart");
end

function RDXM.NAXE.LoathebSpore(arg)
	if string.find(arg, "Fungal Bloom") then
		if loatheb_sporelockout then return; end
		RPC.Invoke("loatheb_spore")
		RDXM.NAXE.LoathebSporeAlert()
	end
end

function RDXM.NAXE.LoathebSporeAlert()
		if loatheb_sporelockout then return; end
		loatheb_sporegroup = loatheb_sporegroup + 1
		if (loatheb_sporegroup < 9) then
			local grouptext = "GROUP " .. loatheb_sporegroup .. " - Go Get Spore!";
			RDX.Alert.Simple(grouptext, nil, 4, false);
		elseif (loatheb_sporegroup == 9) then
			loatheb_sporegroup = 1
			local grouptext = "GROUP " .. loatheb_sporegroup .. " - Go Get Spore!";
			RDX.Alert.Simple(grouptext, nil, 4, false);
		end
		loatheb_sporelockout = true;
		VFL.schedule(2, function() loatheb_sporelockout = false; end);
end


function RDXM.NAXE.LoathebStartDoomTimer()
	-- Show next Impending Doom timer
	loatheb_doomcount = loatheb_doomcount + 1;
	if (loatheb_doomcount < 8) then
		local text = "Doom";
		local consumable = "";
		if (loatheb_doomcount == 2) then
			consumable = " - Nothing";
		elseif (loatheb_doomcount == 3) then
			consumable = " - Bandage";
		elseif (loatheb_doomcount == 4) then
			consumable = " - Healthstone";
		elseif (loatheb_doomcount == 5) then
			consumable = " - Bandage + Pot";
		elseif (loatheb_doomcount == 6) then
			consumable = " - Nothing";
		elseif (loatheb_doomcount == 7) then
			consumable = " - Bandage";
		end

		if (UnitClass("player") == "Warlock") then
			consumable = consumable .. " (AND SHADOW WARD!)";
		end

		text = text .. consumable;

		RDX.Alert.Dropdown("loatheb_doom", text, 40, 5, nil, nil, {r=1, g=.2, b=.2});
		VFL.scheduleExclusive("loatheb_doom", 30, function() RDXM.NAXE.LoathebStartDoomTimer(); end);
	else
		RDX.Alert.Dropdown("loatheb_doom", "15 Sec Dooms - BURN!", 25, 5, "Sound\\Doodad\\BellTollAlliance.wav", nil, {r=1, g=.2, b=.2});
		VFL.scheduleExclusive("loatheb_doom", 15, function() RDXM.NAXE.LoathebStartDoomTimer(); end);
	end
end

function RDXM.NAXE.LoathebStartingHeal()
	if loatheb_healerchannel then
		local channel = GetChannelName(loatheb_healerchannel);
		if (channel) then
			SendChatMessage(UnitName("player") .. " is starting to heal!", "CHANNEL", nil, channel);
		end
	end
end

function RDXM.NAXE.LoathebHealerOrder_RPC(sender, order)
	if sender:IsLeader() then
		for i=1, table.getn(order) do
			if order[i] == UnitName("player") then
				local healspacing = 60 / table.getn(order);
				local healstart = i * healspacing;
				RDX.Alert.Dropdown("loatheb_heal", "Heal!", healstart, 5, "Sound\\Doodad\\BellTollAlliance.wav", nil, {r=1, g=.2, b=.2});
				VFL.scheduleExclusive("loatheb_healstart", healstart, function() RDXM.NAXE.LoathebStartingHeal(); end);
				return;
			end
		end
	end
end
RPC.Bind("loatheb_healerorder", RDXM.NAXE.LoathebHealerOrder_RPC);

function RDXM.NAXE.LoathebHealerChannel_RPC(sender, channel)
	loatheb_healerchannel = channel;
end
RPC.Bind("loatheb_healerchannel", RDXM.NAXE.LoathebHealerChannel_RPC);

function RDXM.NAXE.LoathebUpdate()
	RDX.AutoStartStopEncounter(loatheb_track);
	RDX.AutoUpdateEncounterPane(loatheb_track);
end


RDXM.NAXE.enctbl["loatheb"] = {
	ActivateEncounter = RDXM.NAXE.LoathebActivate;
	DeactivateEncounter = RDXM.NAXE.LoathebDeactivate;
	StartEncounter = RDXM.NAXE.LoathebStart;
	StopEncounter = RDXM.NAXE.LoathebStop;
};